@@ -24,9 +24,11 @@
 int gengetopt_define_package (char * s) ;
 int gengetopt_define_version (char * s) ;
 int gengetopt_define_purpose (char * s) ;
+int gengetopt_add_group (char * s, int required) ;
 int gengetopt_add_option (const char * long_opt, char short_opt, 
                           const char * desc, 
-			  int type, int flagstat, int required,
-                          const char *default_value);
+                          int type, int flagstat, int required,
+                          const char *default_value,
+                          const char * group_value, int multiple);
 
 #endif /* _GENGETOPT_H */
