@@ -4,9 +4,9 @@
  * This file is part of GNU gengetopt 
  *
  * GNU gengetopt is free software; you can redistribute it and/or modify 
- * it under the terms of the GNU General Public License as published by 
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option) 
- * any later version. 
+ * any later version.
  *
  * GNU gengetopt is distributed in the hope that it will be useful, but 
  * WITHOUT ANY WARRANTY; without even the implied warranty of 
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <string>
 
 #include <fstream>
 
@@ -59,6 +60,7 @@ extern "C"
 
 using std::ofstream;
 using std::endl;
+using std::string;
 
 extern char * gengetopt_package;
 extern char * gengetopt_version;
@@ -70,7 +72,9 @@ extern groups_collection_t gengetopt_groups;
 
 static char *create_filename (char *name, char *ext);
 static ofstream *open_fstream (char *filename);
-static char *canonize_names(char * name);
+static char *canonize_names(const char * name);
+static const string strip_path(const string &);
+static const string to_upper(const string &);
 
 char *
 create_filename (char *name, char *ext)
@@ -127,9 +131,11 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
   header_filename = create_filename (filename, header_ext);
 
   // header_gen_class
-  set_header_file_name (filename);
+  const string header_file_name = strip_path (filename);
+  set_header_file_name (header_file_name);
   header_gen_class::set_generator_version (VERSION);
-  set_ifndefname (canonize_names (filename));
+  const string ifndefname = to_upper (strip_path (header_file_name));
+  set_ifndefname (canonize_names (ifndefname.c_str ()));
   header_gen_class::set_parser_name (parser_function_name);
   if (gengetopt_package)
     set_if_package_notdefined (gengetopt_package);
@@ -198,7 +204,7 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
       float_gen.set_structure (ARGS_STRUCT);
     }
 
-  switch (opt->type) 
+  switch (opt->type)
     {
     case ARG_NO:
       stream << "break;";
@@ -347,7 +353,7 @@ CmdlineParserCreator::generate_option_arg(ostream &stream,
         {
           if (opt->flagstat)
             stream << " (default=on)";
-          else    
+          else
             stream << " (default=off)";
         }
       
@@ -545,7 +551,7 @@ CmdlineParserCreator::generate_help_option_print(ostream &stream,
 
   foropt
     {
-      if (opt->group_value && 
+      if (opt->group_value &&
           (! prev_group || strcmp (opt->group_value, prev_group) != 0))
         {
           stream << indent_str;
@@ -742,7 +748,7 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
 }
 
 void
-CmdlineParserCreator::generate_init_unamed(ostream &stream, 
+CmdlineParserCreator::generate_init_unamed(ostream &stream,
                                            unsigned int indent)
 {
   if ( unamed_options )
@@ -775,7 +781,7 @@ CmdlineParserCreator::generate_long_option_struct(ostream &stream,
              << (opt->type == ARG_NO || opt->type == ARG_FLAG ? 0 : 1) 
              << ", NULL, ";
 
-      if (opt->short_opt) 
+      if (opt->short_opt)
         stream << "\'" << opt->short_opt << "\'";
       else 
         stream << "0";
@@ -860,7 +866,7 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
   opt_gen.set_gen_exit (no_handle_error ? "return" : "exit");
   bool first = true;
 
-  foropt 
+  foropt
     if (! opt->short_opt)
       {
         stream << indent_str;
@@ -873,6 +879,16 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
         do_update_arg (opt, str_stream, indent + 2);
         opt_gen.set_update_arg (str_stream.str ());
 
+        generic_option_group_gen_class group_gen;
+        if (opt->group_value)
+          {
+            str_stream.str ("");
+            str_stream << " ";
+            group_gen.set_group_var_name (opt->group_value);
+            group_gen.generate_generic_option_group (str_stream, indent);
+            opt_gen.set_update_group_count (str_stream.str ());
+          }
+
         opt_gen.generate_no_short_option (stream, indent);
 
         stream << endl;
@@ -956,7 +972,7 @@ CmdlineParserCreator::generate_handle_group(ostream &stream,
   string indent_str (indent, ' ');
 
   groups_collection_t::const_iterator end = gengetopt_groups.end();
-  for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin(); 
+  for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin();
         idx != end; ++idx)
     {
       stream << indent_str;
@@ -971,14 +987,14 @@ CmdlineParserCreator::generate_handle_group(ostream &stream,
           opt_gen.set_Comparison_rule(GROUP_NOT_REQUIRED_COMPARISON);
           opt_gen.set_number_required(GROUP_NOT_REQUIRED_MESSAGE);
         }
-		
+
       opt_gen.generate_group_option (stream, indent);
       stream << endl;
     }
 }
 
 void
-CmdlineParserCreator::generate_handle_required(ostream &stream, 
+CmdlineParserCreator::generate_handle_required(ostream &stream,
                                                unsigned int indent)
 {
   struct gengetopt_option * opt;
@@ -1006,7 +1022,7 @@ CmdlineParserCreator::generate_handle_required(ostream &stream,
 }
 
 void
-CmdlineParserCreator::generate_group_counters(ostream &stream, 
+CmdlineParserCreator::generate_group_counters(ostream &stream,
                                               unsigned int indent)
 {
   group_counter_gen_class counter_gen;
@@ -1020,7 +1036,7 @@ CmdlineParserCreator::generate_group_counters(ostream &stream,
       counter_gen.set_group_name (idx->first);
       counter_gen.generate_group_counter (stream, indent);
       stream << endl;
-    }     
+    }
 }
 
 int
@@ -1042,7 +1058,7 @@ CmdlineParserCreator::generate_source ()
 }
 
 void
-CmdlineParserCreator::generate_handle_unamed(ostream &stream, 
+CmdlineParserCreator::generate_handle_unamed(ostream &stream,
                                              unsigned int indent)
 {
   if (! unamed_options)
@@ -1055,7 +1071,7 @@ CmdlineParserCreator::generate_handle_unamed(ostream &stream,
 }
 
 void
-CmdlineParserCreator::generate_conf_parser(ostream &stream, 
+CmdlineParserCreator::generate_conf_parser(ostream &stream,
                                            unsigned int indent)
 {
   if (! conf_parser)
@@ -1073,7 +1089,7 @@ CmdlineParserCreator::generate_conf_parser(ostream &stream,
   '.' are transformed in '_'.
 */
 char *
-canonize_names (char *name)
+canonize_names (const char *name)
 {
   char *pvar;
   char *p;
@@ -1085,3 +1101,32 @@ canonize_names (char *name)
 
   return pvar;
 }
+
+// remove the path from the file name
+const string
+strip_path(const string &s)
+{
+  string::size_type pos_of_sep;
+
+  pos_of_sep = s.rfind("/");
+  if (pos_of_sep == string::npos)
+    pos_of_sep = s.rfind("\\"); // try also with DOS path sep
+
+  if (pos_of_sep == string::npos)
+    return s; // no path
+
+  return s.substr (pos_of_sep + 1);
+}
+
+const string
+to_upper(const string &old)
+{
+  string upper = old;
+
+  for (string::iterator s = upper.begin ();
+       s != upper.end (); ++s)
+    *s = toupper (*s);
+
+  return upper;
+}
+
